<?php

/**
 * @file
 * Contains \Drupal\couchbasedrupal\Cache\CouchbaseBackendFactory.
 */

namespace Drupal\couchbasedrupal\Cache;

use Drupal\Core\Site\Settings;

use Drupal\Core\Cache\CacheFactoryInterface;
use Drupal\Core\Cache\CacheTagsChecksumInterface;

class CouchbaseBackendFactory implements CacheFactoryInterface {

  /**
   * The site prefix string.
   *
   * @var string
   */
  protected $sitePrefix;

  /**
   * The cache tags checksum provider.
   *
   * @var \Drupal\Core\Cache\CacheTagsChecksumInterface
   */
  protected $checksumProvider;


  /**
   * Memcache settings.
   *
   * @var arrary()
   */
  protected $settings;

  /**
   * List of instanced clusters.
   *
   * @var \CouchbaseCluster[]
   */
  protected static $clusters = array();

  /**
   * List of instanced buckets.
   *
   * @var \CouchbaseBucket[]
   */
  protected static $buckets = array();

  /**
   * Retrieve the default couchbase server as set in settings.php
   *
   * @param array $settings
   *   Couchbase settings. Get from Settings::get('couchbase_settings')
   * @throws \Exception
   * @return \CouchbaseCluster|bool
   */
  public static function GetCouchbaseServer($settings, $name = 'default') {

    if (!is_array($settings)) {
      return FALSE;
    }

    // TODO: Connections to the couchbase buckets should be managed in a service...
    // messages should are not properly built :(
    if (!class_exists(\CouchbaseCluster::class)) {
      \Drupal::logger('couchbasedrupal')->notice("CouchbaseCluster class does not exist. Make sure the couchbase extension is up and running.");
      return FALSE;
    }

    if (!isset($settings['servers'][$name]['uri'])) {
      \Drupal::logger('couchbasedrupal')->notice("Couchbase server settings for $name not found");
      return FALSE;
    }

    $uri = $settings['servers']['default']['uri'];

    try {
      $server = static::ParseUri($uri);
    }
    catch (\Exception $e) {
      \Drupal::logger('couchbasedrupal')->notice("Could not parse uri for couchbase server: $uri");
      return FALSE;
    }

    if (!isset(static::$clusters[$uri])) {
      if (isset($server['username']) && isset($server['password'])) {
        static::$clusters[$uri] = new \CouchbaseCluster($server['scheme'] . '://' . $server['host'], $server['username'], $server['password']);
      }
      else {
        static::$clusters[$uri] = new \CouchbaseCluster($server['scheme'] . '://' . $server['host']);
      }
    }

    return static::$clusters[$uri];
  }

  /**
   * Parse a couchbase connection URI.
   *
   * @param string $uri
   */
  public static function ParseUri($url) {
    $info = parse_url($url);
    if (!isset($info['scheme'], $info['host'])) {
      throw new \InvalidArgumentException('Minimum requirement: driver://host');
    }
    $info += array(
      'user' => '',
      'pass' => '',
      'fragment' => '',
    );

    $database = array(
      'scheme' => $info['scheme'],
      'username' => $info['user'],
      'password' => $info['pass'],
      'host' => $info['host'],
    );

    if (isset($info['port'])) {
      $database['port'] = $info['port'];
    }

    return $database;
  }

  /**
   * Constructs an CouchbaseBackendFactory object.
   *
   * @param string $root
   *   The app root.
   * @param string $site_path
   *   The site path.
   * @param \Drupal\Core\Cache\CacheTagsChecksumInterface $checksum_provider
   *   The cache tags checksum provider.
   */
  public function __construct($root, $site_path, CacheTagsChecksumInterface $checksum_provider, $settings = NULL) {
    if (empty($settings)) {
      $this->settings = \Drupal\Core\Site\Settings::get('couchbasedrupal', array());
    }
    else {
      $this->settings = $settings;
    }
    $this->sitePrefix = Settings::getApcuPrefix('couchbase_backend', $root, $site_path);
    $this->checksumProvider = $checksum_provider;
  }


  /**
   * Gets WincacheBackend for the specified cache bin.
   *
   * @param $bin
   *   The cache bin for which the object is created.
   *
   * @return CouchbaseBackend
   *   The cache backend object for the specified cache bin.
   */
  public function get($bin) {

    $cluster = static::GetCouchbaseServer($this->settings);

    $bucketName = 'default';
    $bucket = NULL;
    if (!isset(static::$buckets[$bucketName])) {
      static::$buckets[$bucketName] = $cluster->openBucket('default');
    }
    $bucket = static::$buckets[$bucketName];


    return new CouchbaseBackend($bin, $this->sitePrefix, $this->checksumProvider, $bucket);
  }
}
